#include "ratecond.h"
#include "projections.h"
#include "utils.h"

///////////////
// This function computes the bound on the logarithmic min-norm of a matrix
interval ml(const IMatrix &A)
{
	capd::vectalg::EuclLNorm<IVector,IMatrix> l;
	return -l(-A);
}

bool satisfiesRateConditions(const IMatrix &DF,const interval &L,const int &x_dimension)
{	
	capd::vectalg::EuclLNorm<IVector,IMatrix> l;
	capd::vectalg::EuclNorm <IVector,IMatrix> euclNorm;
	
	IMatrix dF_xx=Fxx(DF,x_dimension);
	IMatrix dF_xy=Fxy(DF,x_dimension);
	IMatrix dF_yx=Fyx(DF,x_dimension);
	IMatrix dF_yy=Fyy(DF,x_dimension);
		
	interval mu=l(dF_yy)+euclNorm(dF_yx)/L;
	interval xi =ml(dF_xx)-L*euclNorm(dF_xy);
	
	if(! (mu<0.0)) return 0;
	if(! (0.0<xi)) return 0;
	return 1;
}

interval expansionRate(const IMatrix &DF,const interval &L,const int &x_dimension)
{
	capd::vectalg::EuclNorm <IVector,IMatrix> euclNorm;
	
	IMatrix dF_xx=Fxx(DF,x_dimension);
	IMatrix dF_xy=Fxy(DF,x_dimension);
	IMatrix dF_yx=Fyx(DF,x_dimension);
	IMatrix dF_yy=Fyy(DF,x_dimension);
		
	interval xi =ml(dF_xx)-L*euclNorm(dF_xy);
	return xi;
}